import argparse
import os
import json
from pathlib import Path
from eduslip.auth_controller import is_user_subscribed
from eduslip.slip_controller import get_slip

ALL_SUBJECTS = ["os", "java", "php", "dsa"]

def main():
    parser = argparse.ArgumentParser(description="Eduslip CLI")
    parser.add_argument("username", help="Your username")
    parser.add_argument("subject", help="Subject (os/java/php)")
    parser.add_argument("slip_number", type=int, help="Slip number")
    args = parser.parse_args()

    subject = args.subject.lower()

    if subject not in ALL_SUBJECTS:
        print(f'Invalid subject "{args.subject}". Available: {", ".join(ALL_SUBJECTS)}')
        return

    # subject ranges
    if subject == "os" and not (1 <= args.slip_number <= 25):
        print("Invalid slip number for OS. Valid range: 1–25")
        return
    if subject == "php" and not (1 <= args.slip_number <= 30):
        print("Invalid slip number for PHP. Valid range: 1–30")
        return
    if subject == "java" and not (1 <= args.slip_number <= 30):
        print("Invalid slip number for JAVA. Valid range: 1–30")
        return
    if subject == "dsa" and not (1 <= args.slip_number <= 25):
        print("Invalid slip number for DSA. Valid range: 1–25")
        return

    # Check subscription
    subscription = is_user_subscribed(args.username)
    if not subscription.get("success"):
        print(f"[ERROR] {subscription.get('message', 'Subscription check failed')}")
        return

    print(f'[INFO] User "{args.username}" subscription validated.')

    # Get slip
    slip = get_slip(subject, args.slip_number)
    if not slip:
        print("[ERROR] Slip not found.")
        return

    # Write files
    home_dir = Path.home()
    for question in slip["questions"]:
        for filename in question["filenames"]:
            code = slip["answers"].get(filename, "")
            file_path = home_dir / filename
            file_path.parent.mkdir(parents=True, exist_ok=True)
            if filename.endswith(".ipynb") and isinstance(code, dict):
                file_path.write_text(json.dumps(code, indent=2), encoding="utf-8")
            else:
                # For .py, .java, .c, .html, .txt, etc.
                file_path.write_text(code, encoding="utf-8")
            print(f"[SUCCESS] File created: {file_path}")



from eduslip.utils.JAVA import JAVA_SLIPS
from eduslip.utils.OS import OS_SLIPS
from eduslip.utils.PHP import PHP_SLIPS
from eduslip.utils.DSA import DSA_SLIPS

def get_slip(subject: str, slip_no: int):
    if subject == "os":
        return OS_SLIPS[slip_no - 1] if slip_no <= len(OS_SLIPS) else {}
    elif subject == "php":
        return PHP_SLIPS[slip_no - 1] if slip_no <= len(PHP_SLIPS) else {}
    elif subject == "java":
        return JAVA_SLIPS[slip_no - 1] if slip_no <= len(JAVA_SLIPS) else {}
    elif subject == "dsa":
        return DSA_SLIPS[slip_no - 1] if slip_no <= len(DSA_SLIPS) else {}
    return {}



DSA_SLIPS = [
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["Q1.c"]
            },
            {
                "question": " ",
                "filenames": ["btree.h"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "Q1.c": """#include <stdio.h>
#include <stdlib.h>
#include "btree.h"
int main()
{
    int ch, n, i, value, cnt;
    struct bst *newnode, *root, *temp;
    root = NULL;
    while (1)
    {
        printf("\n---Binary Search Tree---\n");
        printf("1.Insert\n");
        printf("2.Inorder Traversal (Recursive)\n");
        printf("3.Exit\n");
        printf("Enter your choice:");
        scanf("%d", &ch);
        switch (ch)
        {
        case 1:
            printf("\nHow many nodes to create:");
            scanf("%d", &n);
            for (i = 0; i < n; i++)
            {
                newnode = create();
                printf("\nEnter the node data:");
                scanf("%d", &newnode->data);
                if (root == NULL)
                    root = newnode;
                else
                    insert(root, newnode);
            }
            break;
        case 2:
            printf("\nInorder Traversal=");
            inorder(root);
            break;
        case 3:
            exit(0);
        default:
            printf("\nInvalid Choice\n");
        }
    }
}""",
            "btree.h": """#include <stdio.h>
#include <stdlib.h>

struct bst
{
    int data;
    struct bst *lchild, *rchild;
} node;
int cnt = 0, leafcnt = 0, nleafcnt = 0;

struct bst *create()
{
    struct bst *temp = (struct bst *)malloc(sizeof(struct bst));
    temp->lchild = NULL;
    temp->rchild = NULL;
    return temp;
}

void insert(struct bst *r, struct bst *new1)
{

    if (new1->data < r->data)
    {
        if (r->lchild == NULL)
            r->lchild = new1;
        else
            insert(r->lchild, new1);
    }

    if (new1->data > r->data)
    {
        if (r->rchild == NULL)
            r->rchild = new1;
        else
            insert(r->rchild, new1);
    }
}
void inorder(struct bst *temp)
{
    if (temp != NULL)
    {
        inorder(temp->lchild);
        printf("%d\t", temp->data);
        inorder(temp->rchild);
    }
}""",
            "Q2.c": """#include <stdio.h>
#include <malloc.h>
void create(int a[10][10], int n)
{
    int i, j;
    printf("\n****TYPE 1 FOR YES & 0 FOR NO****\n");
    for (i = 0; i < n; i++)
    {
        for (j = 0; j < n; j++)
        {
            a[i][j] = 0;
            if (i != j)
            {
                printf("\nIs there any edge between %d & %d: ", i + 1, j + 1);
                scanf("%d", &a[i][j]);
            }
        }
    }
}
void inout(int a[10][10], int n)
{
    int i, j;
    int in = 0, out = 0, total = 0;
    printf("\nVertex\t\tIndegree\tOutdegree\tTotal Degree");
    for (i = 0; i < n; i++)
    {
        in = out = 0;
        for (j = 0; j < n; j++)
        {
            in = in + a[j][i];
            out = out + a[i][j];
        }
        total = in + out;
        printf("\n%d\t\t%d\t\t%d\t\t%d", i + 1, in, out, total);
    }
}
int main()
{
    int a[10][10], n;
    printf("\nEnter the no. of vertex: ");
    scanf("%d", &n);
    create(a, n);
    inout(a, n);
}"""
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
    {
        "slipNo": 1,
        "questions": [
            {
                "question": " ",
                "filenames": ["prime.c"]
            },
            {
                "question": "",
                "filenames": ["Q2.c"]
            },
        ],
        "answers": {
            "prime.c": """ """,
            "Q2.c": """ """
        }
    },
]